package ru.dbotthepony.mc.otm.screen;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.ChatFormatting;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.components.Button;
import net.minecraft.client.gui.components.EditBox;
import net.minecraft.client.renderer.GameRenderer;
import net.minecraft.client.resources.sounds.SimpleSoundInstance;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.item.ItemStack;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.MatterTask;
import ru.dbotthepony.mc.otm.capability.PatternState;
import ru.dbotthepony.mc.otm.menu.MatterPanelMenu;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.network.PatternReplicationRequestPacket;

import javax.annotation.Nonnull;
import java.util.List;

import static org.lwjgl.opengl.GL11.GL_ALWAYS;
import static org.lwjgl.opengl.GL11.GL_LESS;

public class MatterPanelScreen extends MatteryScreen<MatterPanelMenu> {
	private static final int modal_width = 213;
	private static final int modal_height = 110;

	private boolean open_task_list = false;

	protected static final ResourceLocation CONTAINER = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/matter_panel.png");

	protected ResourceLocation CONTAINER_BACKGROUND() {
		return CONTAINER;
	}

	public MatterPanelScreen(MatterPanelMenu p_97741_, Inventory p_97742_, Component p_97743_) {
		super(p_97741_, p_97742_, p_97743_);

		imageWidth = 176;
		imageHeight = 187;

		titleLabelY = 5;

		p_97741_.networkTaskWatcher(task -> {
			if (open_task != null && open_task.equals(task)) {
				open_task = task;
			}
		}, task -> {
			if (open_task != null && open_task.equals(task)) {
				closeTask();
			}
		});
	}

	private PatternState open_pattern;
	private MatterTask open_task;
	private EditBox input_amount;

	private Button inc_8;
	private Button inc_64;
	private Button inc_256;

	private Button dec_8;
	private Button dec_64;
	private Button dec_256;

	private Button send_modal;
	private Button cancel_modal;

	private Button cancel_task;
	private Button close_task;

	private PatternState hovered_pattern;
	private MatterTask hovered_task;

	private double scroller_position = 0;

	@Override
	protected void init() {
		super.init();

		int x = (width - modal_width) / 2;
		int y = (height - modal_height) / 2;

		// окно на создание запроса
		int top_level = 15;
		int button_width = 40;
		int right = 5 + button_width;
		int box_width = modal_width - 5 - 5 - 30;

		inc_8 = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.increase_by", 8), (btn) -> onChangeAmountPress(8));
		right += 2 + button_width;

		inc_64 = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.increase_by", 64), (btn) -> onChangeAmountPress(64));
		right += 2 + button_width;

		inc_256 = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.increase_by", 256), (btn) -> onChangeAmountPress(256));
		right = 5 + button_width;

		top_level += 24;

		input_amount = new EditBox(font, x + modal_width - 5 - box_width, y + top_level, box_width, 16, new TranslatableComponent("otm.container.matter_panel.number_input"));
		input_amount.setMaxLength(6);
		input_amount.setValue("1");
		input_amount.visible = false;

		top_level += 20;

		dec_8 = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.decrease_by", 8), (btn) -> onChangeAmountPress(-8));
		right += 2 + button_width;

		dec_64 = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.decrease_by", 64), (btn) -> onChangeAmountPress(-64));
		right += 2 + button_width;

		dec_256 = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.decrease_by", 256), (btn) -> onChangeAmountPress(-256));

		top_level += 24;
		right = 5 + button_width;

		cancel_modal = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.cancel"), (btn) -> closePattern());
		right += 2 + button_width;

		send_modal = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.send"), (btn) -> onSend());

		inc_8.visible = false;
		inc_64.visible = false;
		inc_256.visible = false;

		dec_8.visible = false;
		dec_64.visible = false;
		dec_256.visible = false;

		send_modal.visible = false;
		cancel_modal.visible = false;

		addWidget(input_amount);
		addWidget(inc_8);
		addWidget(inc_64);
		addWidget(inc_256);
		addWidget(dec_8);
		addWidget(dec_64);
		addWidget(dec_256);
		addWidget(send_modal);
		addWidget(cancel_modal);

		// окно на отмену запроса
		right = 5 + button_width;
		top_level = modal_height - 8 - 20;

		close_task = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.close"), (btn) -> closeTask());

		button_width = 90;
		right += 2 + button_width;
		cancel_task = new Button(x + modal_width - right, y + top_level, button_width, 20, new TranslatableComponent("otm.container.matter_panel.cancel_task"), (btn) -> cancelTask());

		close_task.visible = false;
		cancel_task.visible = false;

		addWidget(close_task);
		addWidget(cancel_task);
	}

	private void onChangeAmountPress(int amount) {
		int value = 1;

		try {
			value = Integer.parseInt(input_amount.getValue());
		} catch (Exception ignored) {

		}

		if (value == 1 && amount > 0)
			input_amount.setValue(Integer.toString(amount));
		else
			input_amount.setValue(Integer.toString(Math.max(1, Math.min(99999, value + amount))));
	}

	private void onSend() {
		int value = 1;

		try {
			value = Integer.parseInt(input_amount.getValue());
		} catch (Exception ignored) {

		}

		MatteryNetworking.CHANNEL.sendToServer(new PatternReplicationRequestPacket(open_pattern, value));
		closePattern();
	}

	private void closePattern() {
		open_pattern = null;
		input_amount.visible = false;
		input_amount.changeFocus(false);
		inc_8.visible = false;
		inc_64.visible = false;
		inc_256.visible = false;

		send_modal.visible = false;
		cancel_modal.visible = false;

		dec_8.visible = false;
		dec_64.visible = false;
		dec_256.visible = false;
	}

	private void openTask(MatterTask task) {
		open_task = task;

		close_task.visible = true;
		cancel_task.visible = true;
	}

	private void closeTask() {
		open_task = null;

		close_task.visible = false;
		cancel_task.visible = false;
	}

	private void cancelTask() {
		menu.requestTaskCancel(open_task.id());
		closeTask();
	}

	@Override
	protected void containerTick() {
		super.containerTick();

		input_amount.tick();
	}

	@Override
	public void resize(Minecraft p_96575_, int p_96576_, int p_96577_) {
		var input_amount = this.input_amount;

		super.resize(p_96575_, p_96576_, p_96577_);

		if (open_task != null) {
			openTask(open_task);
		}

		if (open_pattern != null) {
			openPattern(open_pattern);
		}

		if (input_amount != null) {
			this.input_amount.setValue(input_amount.getValue());
		}
	}

	protected void renderForeground(PoseStack stack, int mouseX, int mouseY, float p_98421_) {
		if (open_pattern != null) {
			RenderSystem.disableDepthTest();

			input_amount.render(stack, mouseX, mouseY, p_98421_);

			RenderSystem.enableDepthTest();
			RenderSystem.depthFunc(GL_ALWAYS);

			inc_8.render(stack, mouseX, mouseY, p_98421_);

			inc_64.render(stack, mouseX, mouseY, p_98421_);

			inc_256.render(stack, mouseX, mouseY, p_98421_);

			dec_8.render(stack, mouseX, mouseY, p_98421_);
			dec_64.render(stack, mouseX, mouseY, p_98421_);
			dec_256.render(stack, mouseX, mouseY, p_98421_);
			dec_8.render(stack, mouseX, mouseY, p_98421_);

			send_modal.render(stack, mouseX, mouseY, p_98421_);
			cancel_modal.render(stack, mouseX, mouseY, p_98421_);

			RenderSystem.depthFunc(GL_LESS);
		} else if (open_task != null) {
			RenderSystem.enableDepthTest();
			RenderSystem.depthFunc(GL_ALWAYS);

			cancel_task.render(stack, mouseX, mouseY, p_98421_);
			close_task.render(stack, mouseX, mouseY, p_98421_);

			RenderSystem.depthFunc(GL_LESS);
		}
	}

	@Override
	protected void renderLabels(PoseStack p_97808_, int p_97809_, int p_97810_) {}

	@Nonnull
	@Override
	public List<Component> getTooltipFromItem(ItemStack p_96556_) {
		List<Component> get_list = super.getTooltipFromItem(p_96556_);

		if (hovered_pattern != null) {
			get_list.add(new TranslatableComponent("otm.item.pattern.research", String.format("%.2f", hovered_pattern.research_percent() * 100d)).withStyle(ChatFormatting.AQUA));
		} else if (hovered_task != null) {
			get_list.add(new TranslatableComponent("otm.gui.matter_task.total", hovered_task.total()).withStyle(ChatFormatting.GRAY));
			get_list.add(new TranslatableComponent("otm.gui.matter_task.required", hovered_task.required()).withStyle(ChatFormatting.GRAY));
			get_list.add(new TranslatableComponent("otm.gui.matter_task.in_progress", hovered_task.in_progress()).withStyle(ChatFormatting.GRAY));
			get_list.add(new TranslatableComponent("otm.gui.matter_task.finished", hovered_task.finished()).withStyle(ChatFormatting.GRAY));
		}

		return get_list;
	}

	@Override
	protected void renderTooltip(PoseStack pose, int mouseX, int mouseY) {
		if (open_pattern != null) {
			int x = (width - modal_width) / 2 + 7;
			int y = input_amount.y + 1;

			if (mouseX >= x && mouseX <= x + 16 && mouseY >= y && mouseY <= y + 16) {
				this.renderTooltip(pose, new ItemStack(open_pattern.item(), 1), mouseX, mouseY);
				return;
			}
		} else if (hovered_pattern != null) {
			this.renderTooltip(pose, new ItemStack(hovered_pattern.item(), 1), mouseX, mouseY);
			return;
		} else if (hovered_task != null) {
			this.renderTooltip(pose, new ItemStack(hovered_task.item(), hovered_task.total()), mouseX, mouseY);
			return;
		}

		super.renderTooltip(pose, mouseX, mouseY);
	}

	private void openPattern(PatternState state) {
		open_pattern = state;
		input_amount.setValue("1");
		// input_amount.setFocus(true); // bugged
		scrolling = false;

		input_amount.visible = true;
		inc_8.visible = true;
		inc_64.visible = true;
		inc_256.visible = true;

		dec_8.visible = true;
		dec_64.visible = true;
		dec_256.visible = true;

		send_modal.visible = true;
		cancel_modal.visible = true;
	}

	private double scroller_position_regular = 0;
	private double scroller_position_tasks = 0;

	private void openTaskList() {
		open_task_list = true;
		scroller_position_regular = scroller_position;
		scroller_position = scroller_position_tasks;

	}

	private void closeTaskList() {
		open_task_list = false;
		scroller_position_tasks = scroller_position;
		scroller_position = scroller_position_regular;
	}

	@Override
	public boolean mouseClicked(double mouse_x, double mouse_y, int p_97750_) {
		if (open_task == null && open_pattern == null) {
			if (hovered_pattern != null) {
				openPattern(hovered_pattern);
				Minecraft.getInstance().getSoundManager().play(SimpleSoundInstance.forUI(SoundEvents.UI_BUTTON_CLICK, 1.0F));
				return true;
			} else if (hovered_task != null) {
				openTask(hovered_task);
				Minecraft.getInstance().getSoundManager().play(SimpleSoundInstance.forUI(SoundEvents.UI_BUTTON_CLICK, 1.0F));
				return true;
			}
		}

		int button_x = leftPos + 157;
		int button_y = topPos + 20 + (int) ((scroller_position / maxScroll()) * 145);
		int button_width = 12;
		int button_height = 15;

		scrolling = maxScroll() > 0 && mouse_x >= button_x && button_x + button_width >= mouse_x && mouse_y >= button_y && button_y + button_height >= mouse_y;

		if (open_task == null && open_pattern == null) {
			if (open_task_list) {
				button_x = leftPos;
				button_y = topPos - 28;
				button_width = 28;
				button_height = 30;

				if (mouse_x >= button_x && button_x + button_width >= mouse_x && mouse_y >= button_y && button_y + button_height >= mouse_y) {
					closeTaskList();
					Minecraft.getInstance().getSoundManager().play(SimpleSoundInstance.forUI(SoundEvents.UI_BUTTON_CLICK, 1.0F));
				}
			} else {
				button_x = leftPos + 28;
				button_y = topPos - 28;
				button_width = 28;
				button_height = 30;

				if (mouse_x >= button_x && button_x + button_width >= mouse_x && mouse_y >= button_y && button_y + button_height >= mouse_y) {
					openTaskList();
					Minecraft.getInstance().getSoundManager().play(SimpleSoundInstance.forUI(SoundEvents.UI_BUTTON_CLICK, 1.0F));
				}
			}
		}

		return super.mouseClicked(mouse_x, mouse_y, p_97750_);
	}

	@Override
	public boolean mouseReleased(double p_97812_, double p_97813_, int p_97814_) {
		scrolling = false;
		return super.mouseReleased(p_97812_, p_97813_, p_97814_);
	}

	@Override
	public boolean mouseScrolled(double mouse_x, double mouse_y, double scroll) {
		if (open_pattern != null) {
			return true;
		}

		if (super.mouseScrolled(mouse_x, mouse_y, scroll)) {
			return true;
		}

		scroller_position = Math.max(0, Math.min(maxScroll(), scroller_position - scroll));

		return true;
	}

	@Override
	public boolean mouseDragged(double mouse_x, double mouse_y, int flag, double drag_x, double drag_y) {
		if (scrolling && maxScroll() > 0) {
			if (mouse_y < topPos + 20) {
				scroller_position = 0;
			} else if (mouse_y > topPos + 20 + 160) {
				scroller_position = maxScroll();
			} else {
				scroller_position = maxScroll() * (mouse_y - topPos - 20) / 160d;
			}
		}

		return super.mouseDragged(mouse_x, mouse_y, flag, drag_x, drag_y);
	}

	private boolean scrolling = false;

	private int maxScroll() {
		if (open_task_list) {
			if (menu.tasks.size() < 16) {
				return 0;
			}

			return (int) Math.floor(menu.tasks.size() / 8d - 2d);
		}

		if (menu.patterns.size() < 16) {
			return 0;
		}

		return (int) Math.floor(menu.patterns.size() / 8d - 2d);
	}

	@Override
	protected void renderBg(PoseStack pose, float p_97788_, int mouseX, int mouseY) {
		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, CONTAINER_BACKGROUND());

		if (open_task_list) {
			// список паттернов
			blit(pose, leftPos, topPos - 28, 176, 65, 28, 30);
		} else {
			// список задач
			blit(pose, leftPos + 28, topPos - 28, 204, 65, 28, 30);
		}

		super.renderBg(pose, p_97788_, mouseX, mouseY);

		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, CONTAINER_BACKGROUND());

		if (maxScroll() > 0) {
			if (scrolling) {
				blit(pose, leftPos + 157, topPos + 20 + (int) ((scroller_position / maxScroll()) * 145), 188, 0, 12, 15);
			} else {
				blit(pose, leftPos + 157, topPos + 20 + (int) ((scroller_position / maxScroll()) * 145), 176, 0, 12, 15);
			}
		} else {
			blit(pose, leftPos + 157, topPos + 20, 176, 0, 12, 15);
		}

		if (open_task_list) {
			// список задач
			blit(pose, leftPos + 28, topPos - 28, 204, 33, 28, 32);
		} else {
			// список паттернов
			blit(pose, leftPos, topPos - 28, 176, 33, 28, 32);
		}

		this.font.draw(pose, this.title, (float) (leftPos + this.titleLabelX), (float) (topPos + this.titleLabelY), 4210752);

		hovered_pattern = null;
		hovered_task = null;

		int cells_width = 8;
		int cells_height = 9;

		int render_x = leftPos + 10;
		int render_y = topPos + 20;
		int cell_x = 0;
		int cell_y = 0;

		RenderSystem.enableDepthTest();

		if (open_task_list) {
			for (int index = ((int) scroller_position) * 9; index < menu.tasks.size(); index++) {
				var state = menu.tasks.get(index);
				var itemstack = new ItemStack(state.item(), Math.max(1, state.required()));

				// player, itemstack, x, y, ??????????
				this.itemRenderer.renderAndDecorateItem(this.minecraft.player, itemstack, render_x, render_y, -1);
				// font, itemstack, x, y, string
				this.itemRenderer.renderGuiItemDecorations(this.font, itemstack, render_x, render_y, null);

				if (hovered_task == null && mouseX >= render_x && mouseX <= render_x + 16 && mouseY >= render_y && mouseY <= render_y + 16) {
					hovered_task = state;
				}

				cell_x++;
				render_x += 18;

				if (cell_x >= cells_width) {
					render_x = leftPos + 10;
					render_y += 18;
					cell_x = 0;
					cell_y++;

					if (cell_y >= cells_height)
						break;
				}
			}
		} else {
			for (int index = ((int) scroller_position) * 9; index < menu.patterns.size(); index++) {
				var state = menu.patterns.get(index);
				var itemstack = new ItemStack(state.item(), 1);

				// player, itemstack, x, y, ??????????
				this.itemRenderer.renderAndDecorateItem(this.minecraft.player, itemstack, render_x, render_y, -1);
				// font, itemstack, x, y, string
				this.itemRenderer.renderGuiItemDecorations(this.font, itemstack, render_x, render_y, null);

				if (hovered_pattern == null && mouseX >= render_x && mouseX <= render_x + 16 && mouseY >= render_y && mouseY <= render_y + 16) {
					hovered_pattern = state;
				}

				cell_x++;
				render_x += 18;

				if (cell_x >= cells_width) {
					render_x = leftPos + 10;
					render_y += 18;
					cell_x = 0;
					cell_y++;

					if (cell_y >= cells_height)
						break;
				}
			}
		}

		if (open_pattern != null || open_task != null) {
			RenderSystem.depthFunc(GL_ALWAYS);

			this.renderBackground(pose);

			RenderSystem.setShader(GameRenderer::getPositionTexShader);
			RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
			RenderSystem.setShaderTexture(0, CONTAINER_BACKGROUND());
			// PoseStack, x, y, image_start_x, image_start_y, rect_size_x, rect_size_y

			int x = (width - modal_width) / 2;
			int y = (height - modal_height) / 2;

			// заголовок
			int dy = 0;
			this.blit(pose, x, y, 0, 188, modal_width, 4);
			dy += 4;

			for (; dy < modal_height - 10; dy += 5)
				this.blit(pose, x, y + dy, 0, 192, modal_width, 5);

			this.blit(pose, x, y + dy, 0, 197, modal_width, 4);

			if (open_pattern != null) {
				// фон слота
				this.blit(pose, x + 6, input_amount.y, 176, 15, 18, 18);
			} else {
				this.blit(pose, x + 6, y + 19, 176, 15, 18, 18);
			}

			ItemStack itemstack;

			if (open_pattern != null) {
				itemstack = new ItemStack(open_pattern.item(), 1);
			} else {
				itemstack = new ItemStack(open_task.item(), 1);
			}

			RenderSystem.enableDepthTest();

			if (open_pattern != null) {
				// player, itemstack, x, y, ??????????
				this.itemRenderer.renderAndDecorateItem(this.minecraft.player, itemstack, x + 7, input_amount.y + 1, 100);
				// font, itemstack, x, y, string
				this.itemRenderer.renderGuiItemDecorations(this.font, itemstack, x + 7, input_amount.y + 1, null);
			} else {
				// player, itemstack, x, y, ??????????
				this.itemRenderer.renderAndDecorateItem(this.minecraft.player, itemstack, x + 7, y + 20, 100);
				// font, itemstack, x, y, string
				// this.itemRenderer.renderGuiItemDecorations(this.font, itemstack, x + 7, input_amount.y + 1, null);
			}

			if (open_pattern != null) {
				this.font.draw(pose, new TranslatableComponent("otm.container.matter_panel.label"), (float) (x + 6), (float) (y + 5), 4210752);
				this.font.draw(pose, new TranslatableComponent("otm.item.pattern.line", itemstack.getDisplayName(), String.format("%.2f", open_pattern.research_percent() * 100d)), (float) (x + 6), (float) (cancel_modal.y), ChatFormatting.AQUA.getColor());
			} else {
				this.font.draw(pose, new TranslatableComponent("otm.container.matter_panel.task"), (float) (x + 6), (float) (y + 5), 4210752);
				this.font.draw(pose, new TranslatableComponent("otm.container.matter_panel.task_line", itemstack.getDisplayName(), open_task.in_progress(), open_task.finished(), open_task.total()), (float) (x + 27), (float) (y + 20), 4210752);
				this.font.draw(pose, new TextComponent(open_task.id().toString().substring(0, 8) + "..."), (float) (x + 27), (float) (y + 30), 4210752);
			}

			RenderSystem.depthFunc(GL_LESS);
		}
	}
}
